package schedframe.resources.computing.profiles.load;

import java.util.ArrayList;
import java.util.List;

import org.joda.time.DateTimeUtils;

import schedframe.Parameters;
import schedframe.resources.computing.profiles.energy.MeasurementHistory;
import schedframe.resources.computing.profiles.load.plugin.LoadEstimationPlugin;
import schemas.LoadCalendar;
import schemas.WeekDay;
import schemas.WeekDayTypeChoice;
import schemas.types.DayType;

public class LoadProfile {

	protected List<MeasurementHistory> utlizationHistory;
	protected LoadEstimationPlugin loadEstimationPlugin;

	protected ResourceLoadCalendar loadCalendar;
	
	protected Parameters parameters;
	

	public LoadProfile(LoadEstimationPlugin loadEstimationPlugin, ResourceLoadCalendar loadCal) {
		this.loadEstimationPlugin = loadEstimationPlugin;
		this.loadCalendar = loadCal;
		this.utlizationHistory = new ArrayList<MeasurementHistory>();
	}


	public LoadEstimationPlugin getLoadEstimationPlugin() {
		return loadEstimationPlugin;
	}
	
	public ResourceLoadCalendar getLoadCalendar() {
		if(loadCalendar == null){
			LoadCalendar lc = new LoadCalendar();
			WeekDay weekDay = new WeekDay();
			weekDay.setType(DayType.VALUE_0);
			WeekDayTypeChoice wdc = new WeekDayTypeChoice();
			wdc.setLoadLevel(0);
			weekDay.setWeekDayTypeChoice(wdc);
			lc.addWeekDay(weekDay);	
			return new ResourceLoadCalendar(lc);
		}
		return loadCalendar;
	}

	public boolean addToLoadHistory(double load) {

		if (utlizationHistory.size() == 0) {
			MeasurementHistory usage = new MeasurementHistory(DateTimeUtils.currentTimeMillis(), load);
			utlizationHistory.add(usage);
			return true;
		}

		boolean status = false;
		int lastIdx = utlizationHistory.size() - 1;
		double lastLoad = utlizationHistory.get(lastIdx).getValue();
		if (lastLoad != load) {
			status = true;
			MeasurementHistory util = utlizationHistory.get(lastIdx);
			long currentTime = DateTimeUtils.currentTimeMillis();
			if (util.getTimestamp() == currentTime) {
				util.setValue(load);
				if(lastIdx > 0 && utlizationHistory.get(lastIdx - 1).getValue() == load)
					utlizationHistory.remove(util);
			} else {
				util = new MeasurementHistory(DateTimeUtils.currentTimeMillis(), load);
				utlizationHistory.add(util);
			}
		}
		return status;
	}

	public List<MeasurementHistory> getUtilizationHistory() {
		return utlizationHistory;
	}

	public void init(Parameters params){
		this.parameters = params;
	}
	
	public Parameters getParameters() {
		return parameters;
	}

}
